==> Apache2-ASP-1.58.tar.gz: Apache2-ASP-1.58/lib/Apache2/ASP/ApplicationStateManager.pm <==

package Apache2::ASP::ApplicationStateManager;

use strict;
use warnings 'all';
use base 'Ima::DBI';
use Storable qw( freeze thaw );


#==============================================================================
sub new
{
  my ($class, $asp) = @_;
  
  my $s = bless {
    application_name => $asp->config->application_name,
  }, $class;
  
  local $^W = 0;
  __PACKAGE__->set_db('Apps', 
    $asp->config->application_state->dsn,
    $asp->config->application_state->username,
    $asp->config->application_state->password, {
      RaiseError  => 1,
      AutoCommit  => 1,
    }
  );# unless __PACKAGE__->can('db_Apps');
  
  if( my $res = $s->retrieve )
  {
    return $res;
  }
  else
  {
    return $s->create;
  }# end if()
}# end new()


#==============================================================================
sub create
{
  my $s = shift;
  
  my $sth = $s->dbh->prepare(<<"");
    INSERT INTO asp_applications (
      application_id,
      application_data
    )
    VALUES (
      ?, ?
    )

  $sth->execute(
    $s->{application_name},
    freeze( {} )
  );
  $sth->finish();
  
  return $s->retrieve();
}# end create()


#==============================================================================
sub retrieve
{
  my $s = shift;
  
  my $sth = $s->dbh->prepare(<<"");
    SELECT application_data
    FROM asp_applications
    WHERE application_id = ?

  $sth->execute( $s->{application_name} );
  my ($data) = $sth->fetchrow;
  $sth->finish();
  
  return unless $data;
  
  $data = thaw($data);
  $data->{application_name} = $s->{application_name};
  return bless $data, ref($s);
}# end retrieve()


#==============================================================================
sub save
{
  my $s = shift;
  
  my $sth = $s->dbh->prepare(<<"");
    UPDATE asp_applications SET
      application_data = ?
    WHERE application_id = ?

  my $data = { %$s };
  delete($data->{dbh});
  delete($data->{application_name});
  $sth->execute(
    freeze( $data ),
    $s->{application_name}
  );
  $sth->finish();
  
  1;
}# end save()


#==============================================================================
sub dbh
{
  my $s = shift;
  
  return $s->db_Apps;
}# end dbh()


#==============================================================================
sub DESTROY
{
  my $s = shift;
  delete($s->{$_}) foreach keys(%$s);
}# end DESTROY()


1;# return true:

__END__

=pod

=head1 NAME

Apache2::ASP::ApplicationStateManager - Base class for Application State Managers.

=head1 SYNOPSIS

Within your ASP script:

  <%
    $Application->{counter}++;
    $Response->Write("This website has had $Application->{counter} visitors since restart.");
  %>

=head1 DESCRIPTION

The global C<$Application> object is an instance of a subclass of C<Apache2::ASP::ApplicationStateManager>.

It is a blessed hash that is persisted within a database.  Use it to share information across all requests for
all users.

B<NOTE:> - do not store database connections within the C<$Application> object because they cannot be shared across
different processes/threads at this time.

=head1 METHODS

All methods are overridable, but come with sensible defaults.

=head2 new( $asp )

Returns a new C<Apache2::ASP::ApplicationStateManager> object, using C<$asp>.

C<$asp> should be a valid L<Apache2::ASP> object.

=head2 create( )

Creates a new Application.  Returns a new C<Apache2::ASP::ApplicationStateManager> object.

=head2 retrieve( )

Attempts to retrieve the current Application from the data source specified in the global config.

=head2 save( )

Attempts to save the current Application in the data source specified in the global config.

=head2 dbh( )

Returns a blessed L<DBI> connection to the data source specified in the global config.

=head1 BUGS

It's possible that some bugs have found their way into this release.

Use RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Apache2-ASP> to submit bug reports.

=head1 HOMEPAGE

Please visit the Apache2::ASP homepage at L<http://www.devstack.com/> to see examples
of Apache2::ASP in action.

=head1 AUTHOR

John Drago L<mailto:jdrago_999@yahoo.com>

=head1 COPYRIGHT AND LICENSE

Copyright 2007 John Drago, All rights reserved.

This software is free software.  It may be used and distributed under the
same terms as Perl itself.

=cut

==> Apache2-ASP-2.00_04.tar.gz: Apache2-ASP-2.00_04/lib/Apache2/ASP/ApplicationStateManager.pm <==

package Apache2::ASP::ApplicationStateManager;

use strict;
use warnings 'all';
use Storable qw( freeze thaw );
use DBI;
use Scalar::Util 'weaken';
use base 'Ima::DBI';
use Digest::MD5 'md5_hex';


#==============================================================================
sub new
{
  my ($class, %args) = @_;
  
  my $context = delete($args{context});
  my $s = bless {
    context => $context,
  }, $class;

  my $conn = $s->{context}->config->data_connections->application;
  local $^W = 0;
  __PACKAGE__->set_db('Applications', $conn->dsn,
    $conn->username,
    $conn->password, {
      RaiseError  => 1,
      AutoCommit  => 1,
    }
  );
  
  if( my $res = $s->retrieve )
  {
    return $res;
  }
  else
  {
    return $s->create;
  }# end if()
}# end new()


#==============================================================================
sub context
{
  Apache2::ASP::HTTPContext->current;
}# end context()


#==============================================================================
sub create
{
  my $s = shift;
  
  my $sth = $s->dbh->prepare(<<"");
    INSERT INTO asp_applications (
      application_id,
      application_data
    )
    VALUES (
      ?, ?
    )

  $sth->execute(
    $s->context->config->web->application_name,
    freeze( {__signature => md5_hex("")} )
  );
  $sth->finish();
  
  return $s->retrieve();
}# end create()


#==============================================================================
sub retrieve
{
  my $s = shift;
  
  my $sth = $s->dbh->prepare(<<"");
    SELECT application_data
    FROM asp_applications
    WHERE application_id = ?

  $sth->execute( $s->context->config->web->application_name );
  my ($data) = $sth->fetchrow;
  $sth->finish();
  
  return unless $data;
  
  $data = thaw($data);
  $data->{$_} = delete($s->{$_}) foreach qw/ context dbh /;
  weaken($data->{context});
  undef(%$s);
  $s = bless $data, ref($s);
  
  no warnings 'uninitialized';
  $s->{__signature} = md5_hex(
    join ":",
      map { "$_:$s->{$_}" }
        grep { $_ ne '__signature' } sort keys(%$s)
  );
  
  return $s;
}# end retrieve()


#==============================================================================
sub save
{
  my $s = shift;
  
  no warnings 'uninitialized';
  return if $s->{__signature} eq md5_hex(
    join ":",
      map { "$_:$s->{$_}" }
        grep { $_ ne '__signature' } sort keys(%$s)
  );
  $s->{__signature} = md5_hex(
    join ":",
      map { "$_:$s->{$_}" } 
        grep { $_ ne '__signature' } sort keys(%$s)
  );
  
  my $sth = $s->dbh->prepare(<<"");
    UPDATE asp_applications SET
      application_data = ?
    WHERE application_id = ?

  my $data = { %$s };
  delete($data->{dbh});
  delete($data->{context});
  $sth->execute(
    freeze( $data ),
    $s->{application_name}
  );
  $sth->finish();
  
  1;
}# end save()


#==============================================================================
sub dbh
{
  my $s = shift;
  return $s->db_Applications;
}# end dbh()


#==============================================================================
sub DESTROY
{
  my $s = shift;
  
  eval { $s->{dbh}->disconnect } if $s->{dbh};
  delete($s->{$_}) foreach keys(%$s);
}# end DESTROY()

1;# return true:
